from toml_adapt.DocumentOperations import DocumentOperationsEnum
from toml_adapt.DocumentType import DocumentTypeEnum
from toml_adapt.Match import FindMatch
from toml_adapt.AllShapes import all_shapes
from toml_adapt.Shape import Shape
from toml_adapt.ToolExtraction import IsFileCargo, IsFileFlit, IsFileJuliaPkg, IsFilePoetry

def ExtractTool(toml_file_path: str) -> DocumentTypeEnum:    
    options_list=[
        IsFileFlit(toml_file_path),     DocumentTypeEnum.FLIT,
        IsFilePoetry(toml_file_path),   DocumentTypeEnum.POETRY,
        IsFileCargo(toml_file_path),    DocumentTypeEnum.CARGO,
        IsFileJuliaPkg(toml_file_path), DocumentTypeEnum.JULIAPKG]
    try:
        return FindMatch(options_list,lambda check: check)
    except:
        raise Exception("Invalid document type.")

def GetMatchingShape(toml_file_path:str):
    for shape in all_shapes:
        if(shape.DocumentType == ExtractTool(toml_file_path)):
            return shape
    raise Exception("Document type was not found.")

def DoOperation(
    operation: DocumentOperationsEnum,
    toml_file_path: str = "pyproject.toml",
    dependency_name: str = "",
    dependency_version: str = ""
):
    """
    Operation enum describes what operation should be done.
    Toml file path is relative or absolute path to the toml file that should be manipulated.
    Dependency name is the name of dependency that we wish to work with.
    Dependency version is it's version.
    
    When doing remove operation, the version can (and should) be omitted.
    """
    shape: Shape=GetMatchingShape(toml_file_path)
    if(operation==DocumentOperationsEnum.ADD):
        shape.AddDependencyFn(toml_file_path,dependency_name,dependency_version)
    if(operation==DocumentOperationsEnum.REMOVE):
        shape.RemoveDependencyFn(toml_file_path,dependency_name)
    if(operation==DocumentOperationsEnum.CHANGE):
        shape.ChangeDependencyFn(toml_file_path,dependency_name,dependency_version)
    if(operation==DocumentOperationsEnum.ADD_DEV):
        shape.AddDevDependencyFn(toml_file_path,dependency_name,dependency_version)
    if(operation==DocumentOperationsEnum.REMOVE_DEV):
        shape.RemoveDevDependencyFn(toml_file_path,dependency_name)
    if(operation==DocumentOperationsEnum.CHANGE_DEV):
        shape.ChangeDevDependencyFn(toml_file_path,dependency_name,dependency_version)



import pathlib
from typing import Any, Dict, MutableMapping
import toml

def IsSpecificPythonTool(toml_file_path: str, tool_name:str):
    toml_loaded: MutableMapping[str,Any]=toml.load(toml_file_path)
    if(toml_loaded.keys().__contains__("tool")):
        toml_tool: Dict[str,Any]=toml_loaded["tool"]
        if(toml_tool.keys().__contains__(tool_name)):
            return True
    return False

def IsFileFlit(toml_file_path: str) -> bool:
    return IsSpecificPythonTool(toml_file_path,"flit")

def IsFilePoetry(toml_file_path: str) -> bool:
    return IsSpecificPythonTool(toml_file_path,"poetry")

def IsFileCargo(toml_file_path: str) -> bool:
    """
    Cargo requires the file to be named Cargo.toml
    Checking only file path should not be ambiguous
    """
    return pathlib.Path(toml_file_path).name == "Cargo.toml"

def IsFileJuliaPkg(toml_file_path: str) -> bool:
    """
    Julia's pkg tool uses Project.toml file.
    """
    return pathlib.Path(toml_file_path).name == "Project.toml"


from typing import Any, MutableMapping

from toml_adapt.Common import TomlBaseManipulation


def AddPoetryDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    if dependency_version=="X":
        dependency_version="*"
    def poetry(toml_full_dict: MutableMapping[str,Any]):
        toml_full_dict["tool"]["poetry"]["dependencies"][dependency_name]=f"{dependency_version}"
    TomlBaseManipulation(toml_file_path,poetry)
    
def RemovePoetryDependency(toml_file_path:str,
                        dependency_name: str):
    def poetry(toml_full_dict: MutableMapping[str,Any]):
        toml_full_dict["tool"]["poetry"]["dependencies"].pop(dependency_name)
    TomlBaseManipulation(toml_file_path,poetry)
    
def ChangePoetryDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    """
    Same as adding
    """
    if dependency_version=="X":
        dependency_version="*"
    if(dependency_name=="ALL"):
        def poetry(toml_full_dict: MutableMapping[str,Any]):
            all_keys: list[str]=toml_full_dict["tool"]["poetry"]["dependencies"].keys()
            for key in all_keys:
                toml_full_dict["tool"]["poetry"]["dependencies"][key]=f"{dependency_version}"
        TomlBaseManipulation(toml_file_path,poetry)
        return
    
    AddPoetryDependency(toml_file_path,dependency_name,dependency_version)
    
def AddPoetryDevDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    if dependency_version=="X":
        dependency_version="*"
    def poetry(toml_full_dict: MutableMapping[str,Any]):
        toml_full_dict["tool"]["poetry"]["dev-dependencies"][dependency_name]=f"{dependency_version}"
    TomlBaseManipulation(toml_file_path,poetry)
    
def RemovePoetryDevDependency(toml_file_path:str,
                        dependency_name: str):
    def poetry(toml_full_dict: MutableMapping[str,Any]):
        toml_full_dict["tool"]["poetry"]["dev-dependencies"].pop(dependency_name)
    TomlBaseManipulation(toml_file_path,poetry)
    
def ChangePoetryDevDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    """
    Same as adding
    """
    if dependency_version=="X":
        dependency_version="*"
    if(dependency_name=="ALL"):
        def poetry(toml_full_dict: MutableMapping[str,Any]):
            all_keys: list[str]=toml_full_dict["tool"]["poetry"]["dev-dependencies"].keys()
            for key in all_keys:
                toml_full_dict["tool"]["poetry"]["dev-dependencies"][key]=f"{dependency_version}"
        TomlBaseManipulation(toml_file_path,poetry)
        return
    AddPoetryDevDependency(toml_file_path,dependency_name,dependency_version)


from toml_adapt.CargoActions import AddCargoDependency, AddCargoDevDependency, ChangeCargoDependency, ChangeCargoDevDependency, RemoveCargoDependency, RemoveCargoDevDependency
from toml_adapt.DocumentType import DocumentTypeEnum
from toml_adapt.FlitActions import AddFlitDependency, ChangeFlitDependency, RemoveFlitDependency
from toml_adapt.PoetryActions import AddPoetryDependency, AddPoetryDevDependency, ChangePoetryDependency, ChangePoetryDevDependency, RemovePoetryDependency, RemovePoetryDevDependency
from toml_adapt.Shape import Shape

all_shapes: list[Shape]=[
    Shape(DocumentTypeEnum.POETRY,
    AddPoetryDependency,
    RemovePoetryDependency,
    ChangePoetryDependency,
    AddPoetryDevDependency,
    RemovePoetryDevDependency,
    ChangePoetryDevDependency)
    ,
    Shape(DocumentTypeEnum.CARGO,
    AddCargoDependency,
    RemoveCargoDependency,
    ChangeCargoDependency,
    AddCargoDevDependency,
    RemoveCargoDevDependency,
    ChangeCargoDevDependency)
    ,
    Shape(DocumentTypeEnum.FLIT,
    AddFlitDependency,
    RemoveFlitDependency,
    ChangeFlitDependency,
    AddFlitDependency,
    RemoveFlitDependency,
    ChangeFlitDependency)
]



from typing import Callable

from toml_adapt.DocumentType import DocumentTypeEnum

class Shape:
    def __init__(self,
                 document_type: DocumentTypeEnum,
                 add_dependency_function: Callable[[str,str,str],None],
                 remove_dependency_function: Callable[[str,str],None],
                 change_dependency_function: Callable[[str,str,str],None],
                 add_dev_dependency_function: Callable[[str,str,str],None],
                 remove_dev_dependency_function: Callable[[str,str],None],
                 change_dev_dependency_function: Callable[[str,str,str],None]) -> None:
        self.DocumentType: DocumentTypeEnum=document_type
        self.AddDependencyFn: Callable[[str,str,str],None]=add_dependency_function
        self.RemoveDependencyFn: Callable[[str,str],None]=remove_dependency_function
        self.ChangeDependencyFn: Callable[[str,str,str],None]=change_dependency_function
        self.AddDevDependencyFn: Callable[[str,str,str],None]=add_dev_dependency_function
        self.RemoveDevDependencyFn: Callable[[str,str],None]=remove_dev_dependency_function
        self.ChangeDevDependencyFn: Callable[[str,str,str],None]=change_dev_dependency_function



from typing import Any, MutableMapping

from toml_adapt.Common import TomlBaseManipulation


def AddCargoDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    def cargo(toml_full_dict: MutableMapping[str,Any]):
        toml_full_dict["dependencies"][dependency_name]=f"{dependency_version}"
    TomlBaseManipulation(toml_file_path,cargo)
    
def RemoveCargoDependency(toml_file_path:str,
                        dependency_name: str):
    def cargo(toml_full_dict: MutableMapping[str,Any]):
        toml_full_dict["dependencies"].pop(dependency_name)
    TomlBaseManipulation(toml_file_path,cargo)
    
def ChangeCargoDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    """
    Same as adding
    """
    AddCargoDependency(toml_file_path,dependency_name,dependency_version)
    

def AddCargoDevDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    def cargo(toml_full_dict: MutableMapping[str,Any]):
        toml_full_dict["dev-dependencies"][dependency_name]=f"{dependency_version}"
    TomlBaseManipulation(toml_file_path,cargo)
    
def RemoveCargoDevDependency(toml_file_path:str,
                        dependency_name: str):
    def cargo(toml_full_dict: MutableMapping[str,Any]):
        toml_full_dict["dev-dependencies"].pop(dependency_name)
    TomlBaseManipulation(toml_file_path,cargo)
    
def ChangeCargoDevDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    """
    Same as adding
    """
    AddCargoDevDependency(toml_file_path,dependency_name,dependency_version)



from typing import Any, MutableMapping
from toml_adapt.Common import TomlBaseManipulation

def DependencyNameFromFullString(full_string: str)->str:
    """
    Returns the part without version, only dependency name
    """
    return full_string.split()[0]

def AddFlitDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    """
    [tool.flit.metadata]
    """
    def flit(toml_full_dict: MutableMapping[str,Any]):
        list_of_deps: MutableMapping[str,list[str]]=toml_full_dict["tool"]["flit"]["metadata"]
        if len(list(filter(lambda dep: dependency_name == DependencyNameFromFullString(dep),list_of_deps["requires"]))) > 0:
            print("Dependency already exists")
        else:
            toml_full_dict["tool"]["flit"]["metadata"]["requires"].append(f"{dependency_name} {dependency_version}")
    TomlBaseManipulation(toml_file_path,flit)
    
def RemoveFlitDependency(toml_file_path:str,
                        dependency_name: str):
    """
    [tool.flit.metadata]
    """
    def flit(toml_full_dict: MutableMapping[str,Any]):
        list_of_deps: MutableMapping[str,list[str]]=toml_full_dict["tool"]["flit"]["metadata"]
        list_of_deps["requires"]=list(filter(lambda dep: dependency_name != DependencyNameFromFullString(dep),list_of_deps["requires"]))
    TomlBaseManipulation(toml_file_path,flit)
    
def ChangeFlitDependency(toml_file_path:str,
                        dependency_name: str,
                        dependency_version:str):
    """
    [tool.flit.metadata]
    """
    if(dependency_name=="ALL"):
        def flit(toml_full_dict: MutableMapping[str,Any]):
            list_of_deps: MutableMapping[str,list[str]]=toml_full_dict["tool"]["flit"]["metadata"]
            list_of_deps["requires"]=list(map(lambda dep: f"{DependencyNameFromFullString(dep)} {dependency_version}",list_of_deps["requires"]))
        TomlBaseManipulation(toml_file_path,flit)
        return
    def flit(toml_full_dict: MutableMapping[str,Any]):
        list_of_deps: MutableMapping[str,list[str]]=toml_full_dict["tool"]["flit"]["metadata"]
        list_of_deps["requires"]=list(map(lambda dep: f"{dependency_name} {dependency_version}" if dependency_name == DependencyNameFromFullString(dep) else dep,list_of_deps["requires"]))
    TomlBaseManipulation(toml_file_path,flit)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['toml_adapt']

package_data = \
{'': ['*']}

install_requires = \
['click', 'toml']

entry_points = \
{'console_scripts': ['toml-adapt = toml_adapt.main:TomlAdapt']}

setup_kwargs = {
    'name': 'toml-adapt',
    'version': '0.2.3',
    'description': 'A very simple cli for manipulating toml files.',
    'long_description': None,
    'author': 'iztokf',
    'author_email': 'iztokf@fedoraproject.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Callable


def FindMatch(list_of_options: list[Any], predicate: Callable[[Any],bool]):
    for index in range(0,list_of_options.__len__() - 1,2):
        if(predicate(list_of_options[index])):
            return list_of_options[index+1]
    raise Exception("Did not find match")
    


from toml_adapt.CargoActions import AddCargoDependency, AddCargoDevDependency, ChangeCargoDependency, ChangeCargoDevDependency, RemoveCargoDependency, RemoveCargoDevDependency
from toml_adapt.FlitActions import AddFlitDependency, ChangeFlitDependency, RemoveFlitDependency
from toml_adapt.PoetryActions import AddPoetryDependency, AddPoetryDevDependency, ChangePoetryDependency, ChangePoetryDevDependency, RemovePoetryDependency, RemovePoetryDevDependency
from toml_adapt.Shape import Shape
from toml_adapt.DocumentOperations import *
from toml_adapt.Common import ManipulateFile, TomlBaseManipulation

__all__ = [
    "AddCargoDependency",
    "AddCargoDevDependency",
    "ChangeCargoDependency",
    "ChangeCargoDevDependency",
    "RemoveCargoDependency",
    "RemoveCargoDevDependency",
    "AddFlitDependency",
    "ChangeFlitDependency",
    "RemoveFlitDependency",
    "AddPoetryDependency",
    "AddPoetryDevDependency",
    "ChangePoetryDependency",
    "ChangePoetryDevDependency",
    "RemovePoetryDependency",
    "RemovePoetryDevDependency",
    "Shape",
    "ManipulateFile",
    "TomlBaseManipulation",
]

__project__ = "toml_adapt"
__version__ = '0.2.3'



from typing import Any, Callable, MutableMapping
import toml

def ManipulateFile(toml_file_path:str,
                   new_toml: MutableMapping[Any,Any]):
    try:
        with open(toml_file_path, "w") as file:
            toml.dump(new_toml,file)
    except:
        pass

def TomlBaseManipulation(toml_file_path: str,ManipulationFunction: Callable[[Any],None]):
    try:
        toml_full_dict=toml.load(toml_file_path)
        ManipulationFunction(toml_full_dict)
        ManipulateFile(toml_file_path,toml_full_dict)
    except:
        pass


from toml_adapt.DocumentOperations import DocumentOperationsEnum
from toml_adapt.ManipulateToml import DoOperation
import click

@click.command()
@click.option('-path',  help='Path to toml file.', required=True)
@click.option('-a',     help='Action. Any of the following: add, remove, change, add-dev, remove-dev, change-dev.', default='change', required=True)      
@click.option('-dep',   help='Dependency name.', required=True)
@click.option('-ver',   help='Dependency version.',   default="1.0.0")
def TomlAdapt(path: str,a: str,dep:str,ver: str):
    operation: DocumentOperationsEnum=DocumentOperationsEnum.CHANGE
    if(a=='add'):
        operation=DocumentOperationsEnum.ADD
    if(a=='remove'):
        operation=DocumentOperationsEnum.REMOVE
    if(a=='change'):
        operation=DocumentOperationsEnum.CHANGE
    if(a=='add-dev'):
        operation=DocumentOperationsEnum.ADD_DEV
    if(a=='remove-dev'):
        operation=DocumentOperationsEnum.REMOVE_DEV
    if(a=='change-dev'):
        operation=DocumentOperationsEnum.CHANGE_DEV
    DoOperation(operation,
                path,
                dep,
                ver)


if __name__ == "__main__":
    TomlAdapt()